#!/usr/bin/env python
"""RGBPicker displays a window with a list of coloured blocks for each colour in the X11 RGB colour-name database"""

from os import *
from string import *
from gnome.ui import *
from gtk import *
import GDK

class RGBPicker:
    def __init__(self):
        self.win = GtkWindow()

        # Set some 'global' constants
        self.width = 300
        self.height = 300

        # Read in the X11 RGB data
        self.readRGBFile()

        # Set window properties
        self.win.connect("destroy", mainquit)
        self.win.set_title("RGB Picker")

        # Create VBox to hold canvas, label and quit button
        vbox = GtkVBox()
        self.win.add(vbox)
        vbox.show()

        # Create a GtkScrolledWindow to handle scrolling the canvas
        self.canvasScroller = GtkScrolledWindow()
        # Create GnomeCanvas to display the list of colours
        self.canvas = GnomeCanvas()
        self.canvas.set_usize(self.width, self.height)
        self.initialiseCanvas()
        self.canvasScroller.add(self.canvas)
        self.canvas.show()
        vbox.pack_start(self.canvasScroller)
        self.canvasScroller.show()

        # Create GtkLabel to display the selected colour's name and RGB values
	self.label = GtkLabel("")
	vbox.pack_start(self.label, expand=FALSE)
	self.label.show()

        # Create Quit button
        b = GtkButton("Quit")
        b.connect("clicked", mainquit)
        vbox.pack_start(b, expand=FALSE)
        b.show()
        return

    def show(self):
        self.win.show()
        return

    def readRGBFile(self):
        # Execute 'showrgb' and get a pipe file object for reading it's output
        rgbPipe = popen("showrgb")
        rgbText = rgbPipe.read()
        rgbPipe.close()

        # Split the string read from 'showrgb' into a list of lines
        rgbLines = split(rgbText, "\n")

        # Create our list of RGB entries
        self.rgbList = []
        rgbCount = 0

        for rgbLine in rgbLines:
            # Split each line at a double tab (separates RGB values from name)
            rgb = split(rgbLine, "\t\t")
            # If this line actually ahd something on it
            if len(rgb) > 1:
                # Split the RGB values into seperate items
                rgbValues = split(rgb[0])
                rgbName = rgb[1]
                # Construct a dictionary object with the various values
                rgb = { "index": rgbCount,
                        "r": int(rgbValues[0]),
                        "g": int(rgbValues[1]),
                        "b": int(rgbValues[2]),
                        "name": rgbName }
                # Add the dictionary object to our list of RGB entries
                self.rgbList.append(rgb)
                rgbCount = rgbCount + 1

        return

    def initialiseCanvas(self):
        # For each colour in our list...
        for rgb in self.rgbList:
            # Calculate the position of the colour block
            # Each block is 10 units high and 15 units apart vertically (top to top)
            # i.e. a 5 units gap between blocks
            rgbIndex = rgb["index"]
            x1 = 1
            x2 = self.width - 2
            y1 = (rgbIndex * 15) + 1
            y2 = (rgbIndex * 15) + 11
            # Create a new GnomeCanvasRect for this colour
            newItem = self.canvas.root().add("rect", x1=x1, y1=y1, x2=x2, y2=y2, 
                                             fill_color=rgb["name"], outline_color="black",
                                             width_units=1.0)
            # Store the RGB entry for this colour in the GnomeCanvasRect's object data
            newItem.set_data("rgb", rgb)
            # Connect all events from this GnomeCanvasRect to our event handler function
            newItem.connect("event", self.colorItemEvent)

        # Set the scroll region of the canvas to encompass all the colour blocks
        self.canvas.set_scroll_region(0,0, self.width, (len(self.rgbList) * 15) + 10)
        # Set the GtkScrolledWindow's vertical adjustment to the distance between
        # the colour blocks so that scrolling is not too slow
        self.canvasScroller.set_vadjustment(GtkAdjustment(step_incr=15))
        return

    def colorItemEvent(self, widget, event=None):
        if event.type == GDK.ENTER_NOTIFY:
            # Make the outline wide.
            widget.set(width_units=2)
            # Set the label text
            rgb = widget.get_data("rgb")
            labelText = "%(name)s = R: %(r)d  G: %(g)d  B: %(b)d" % rgb
            self.label.set_text(labelText)
            return TRUE
        elif event.type == GDK.LEAVE_NOTIFY:
            # Make the outline thin.
            widget.set(width_units=1)
            # Set the label text
            self.label.set_text("")
            return TRUE

        return FALSE

if __name__ == '__main__':
    win = RGBPicker()
    win.show()
    mainloop()
